@@ -1,4 +1,4 @@
-# $Id: TableExtract.pm,v 1.20 2002/03/11 15:53:29 wsnyder Exp $
+# $Revision: #2 $$Date: 2002/12/13 $$Author: wsnyder $
 # Author: Wilson Snyder <wsnyder@wsnyder.org>
 ######################################################################
 #
@@ -6,9 +6,7 @@
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of either the GNU General Public License or the
-# Perl Artistic License, with the exception that it cannot be placed
-# on a CD-ROM or similar media for commercial distribution without the
-# prior approval of the author.
+# Perl Artistic License.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,10 +21,11 @@
 package SystemC::Vregs::TableExtract;
 
 @ISA = qw(HTML::TableExtract);
-$VERSION = '1.210';
+$VERSION = '1.240';
 
 use strict;
 use vars qw($Debug %Find_Start_Headers %Find_Headers);
+use IO::File;
 use HTML::TableExtract;
 
 %Find_Start_Headers = (Class=>1,
